/**
 * @file Ce fichier gère les requêtes LDAP de type données ; liste des groupe d'un individu, liste des membres d'un groupe... A ne pas confondre avec ldap_auth qui lui gère l'authentification.
 * @author hawkspar
*/
import {ldapescape} from 'ldap-escape';
import ldap from 'ldapjs';
import fs from 'fs';

var ensureLoggedin =  require('connect-ensure-login').ensureLoggedIn;

let config = JSON.parse(fs.readFileSync('config.json', 'utf8'));

var client = ldap.createClient({ url: config.ldap.server, timeout: 10000, idleTimeout: 10000});

/**
 * @summary Fonction qui retrouve les groupes du LDAP dont un individu est membre (pas de bind nécessaire)
 * @arg {int} uid - Identifiant de l'individu à interroger
 * @return {string} Liste des uid de groupes (noms flat des groupes) où l'id fourni
 */
function listGroups(uid) {
    return new Promise(function(resolve, reject) {
        var groupsList=[];
        
        client.search(config.ldap_data_gl.searchBase, {scope: "sub", attributes: "uid", filter: ldapescape.filter(config.ldap_data_gl.searchFilter, {id: uid})}, function(err, res) {
            if (err) {
                reject(err);
            } else {
                res.on('searchEntry', function(entry) { groupsList.push(entry.object.uid); });
                res.on('end', function(res) { resolve(groupsList); });
            }
        });
    });
}
 
/**
 * @summary Fonction qui retrouve la liste des membres d'un binet sur le LDAP
 * @arg {int} gid - Identifiant du groupe à interroger
 * @return {string} Liste des uid de groupes où l'id fournie est membre (noms flat des groupes)
 */
function listMembers(gid) {    
    return new Promise(function(resolve, reject) {
        var membersList=[];
        
        client.search(config.ldap_data_ml.searchBase, {scope: "sub", attributes: "uid", filter: ldapescape.filter(config.ldap_data_ml.searchFilter, {id: gid})}, function(err, res) {
            if (err) {
                reject(err);
            } else {
                res.on('searchEntry', function(entry) { membersList.push(entry.object.restrictedMemberUid); });
                res.on('end', function(res) { resolve(membersList); });
            }
        });
    });
}

// Synthaxe d'utilisation
listGroups("quentin.chevalier").then((grList) => { console.log(grList); });
listMembers("faerix").then((meList) => { console.log(meList); });